@@ -9,7 +9,7 @@ require DynaLoader;
 use vars qw($VERSION @ISA $__import @preload $pid);
 @ISA = qw(DynaLoader);
 sub dl_load_flags { 0x00 }
-$VERSION = '1.64_3';
+$VERSION = '1.65_1';
 $pid = $$;
 bootstrap Prima $VERSION;
 unless ( UNIVERSAL::can('Prima', 'init')) {
@@ -368,16 +368,12 @@ L<Prima::PodView> - POD browser widget
 
 L<Prima::ScrollBar> - scroll bars
 
-L<Prima::ScrollWidget> - scrollable generic document widget
-
 L<Prima::Sliders> - sliding bars, spin buttons and input lines, dial widget etc.
 
 L<Prima::Spinner> - spinner animation
 
 L<Prima::TextView> - rich text browser widget
 
-L<Prima::Widgets> - miscellaneous widget classes
-
 =item Standard dialogs
 
 L<Prima::Dialog::ColorDialog> - color selection facilities
@@ -434,6 +430,28 @@ L<Prima::PS::PDF> - PDF interface to C<Prima::Drawable>
 
 L<Prima::PS::Printer> - PostScript and PDF interfaces to C<Prima::Printer>
 
+=item Widget helpers
+
+L<Prima::Widget::BidiInput> - heuristics for i18n input
+
+L<Prima::Widget::GroupScroller> - optional automatic scroll bars
+
+L<Prima::Widget::IntIndents> - indenting support
+
+L<Prima::Widget::ListBoxUtils> - common paint routine for listboxes
+
+L<Prima::Widget::MouseScroller> - auto repeating mouse events
+
+L<Prima::Widget::Panel> - simple panel widget
+
+L<Prima::Widget::RubberBand> - draw rubberbands
+
+L<Prima::Widget::ScrollWidget> - scrollable generic document widget
+
+L<Prima::Widget::StartupWindow> - a simplistic startup banner window
+
+L<Prima::Widget::UndoActions> - undo and redo the content of editable widgets
+
 =item C interface to the toolkit
 
 L<Prima::internals> - Internal architecture
@@ -454,10 +472,6 @@ L<Prima::Image::Animate> - animate gif and webp files
 
 L<Prima::IniFile> - support of Windows-like initialization files
 
-L<Prima::IntUtils> - internal functions
-
-L<Prima::StartupWindow> - a simplistic startup banner window
-
 L<Prima::StdBitmap> - shared access to the standard toolkit bitmaps
 
 L<Prima::Stress> - stress test module
