@@ -11,20 +11,35 @@ use Prima::Const;
 
 package Prima::array;
 use base 'Tie::Array';
+use Carp;
 
 sub new
 {
-	my ($class, $letter) = @_;
-	die "bad array type" if $letter !~ /^[id]$/;
+	my ($class, $letter, $buf) = @_;
+	die "bad array type" if $letter !~ /^[idSs]$/;
 	my @tie;
+	my @push;
 	my $size = length pack $letter, 0;
-	my $buf = '';
+	if ( defined $buf ) {
+		if ( ref $buf ) {
+			croak "$buf is not an array" unless ref $buf eq 'ARRAY';
+			@push = @$buf;
+			$buf = '';
+		} else {
+			croak "Bad length ". length($buf). ", must be mod $size" if length($buf) % $size;
+		}
+	} else {
+		$buf = '';
+	}
 	tie @tie, $class, $buf, $size, $letter;
+	push @tie, @push if @push;
 	return \@tie;
 }
 
-sub new_int    { shift->new('i') }
-sub new_double { shift->new('d') }
+sub new_short  { shift->new('s', @_) }
+sub new_ushort { shift->new('S', @_) }
+sub new_int    { shift->new('i', @_) }
+sub new_double { shift->new('d', @_) }
 
 use constant REF  => 0;
 use constant SIZE => 1;
@@ -32,26 +47,129 @@ use constant PACK => 2;
 
 sub is_array { ((ref tied @{$_[0]}) // '') eq 'Prima::array' }
 
+sub substr
+{
+	my ( $self, $offset, $length, $replacement) = @_;
+	my $a1 = tied @$self;
+	my $len = length($a1->[REF]) / $a1->[SIZE];
+	croak "offset beyond array boundaries" if $offset > $len || -$offset > $len;
+	my $newref;
+	if ( defined $replacement ) {
+		croak "bad length" if $length < 0;
+		croak "bad array" unless is_array($replacement);
+		my $a2 = tied @$replacement;
+		croak "replacement of type '$a2->[PACK]' is incompatible with type '$a1->[PACK]'"
+			if $a1->[PACK] ne $a2->[PACK];
+		$newref = CORE::substr( $a1->[REF], $offset * $a1->[SIZE], $length * $a1->[SIZE], $a2->[REF]);
+	} elsif ( defined $length ) {
+		$newref = CORE::substr( $a1->[REF], $offset * $a1->[SIZE], $length * $a1->[SIZE]);
+	} else {
+		$newref = CORE::substr( $a1->[REF], $offset * $a1->[SIZE]);
+	}
+	return ref($a1)->new( $a1->[PACK], $newref );
+}
+
 sub append
 {
-	die "bad array" if grep { !is_array($_) } @_;
+	croak "bad array" if grep { !is_array($_) } @_;
 	my ( $a1, $a2 ) = map { tied @$_ } @_;
-	die "bad array" if $a1->[PACK] ne $a2->[PACK];
+	croak "bad array type='$a2->[PACK]', expected '$a1->[PACK]'" if $a1->[PACK] ne $a2->[PACK];
 	$a1->[REF] .= $a2->[REF];
 }
 
+sub clone
+{
+	my $self = tied @{$_[0]};
+	my ( $buf, $size, $pack ) = @$self;
+	return __PACKAGE__->new($pack, $buf);
+}
+
 sub TIEARRAY  { bless \@_, shift }
-sub FETCH     { unpack( $_[0]->[PACK], substr( $_[0]->[REF], $_[1] * $_[0]->[SIZE], $_[0]->[SIZE] )) }
-sub STORE     { substr( $_[0]->[REF], $_[1] * $_[0]->[SIZE], $_[0]->[SIZE], pack( $_[0]->[PACK], $_[2] )) }
+sub FETCH     { unpack( $_[0]->[PACK], CORE::substr( $_[0]->[REF], $_[1] * $_[0]->[SIZE], $_[0]->[SIZE] )) }
+sub STORE     { CORE::substr( $_[0]->[REF], $_[1] * $_[0]->[SIZE], $_[0]->[SIZE], pack( $_[0]->[PACK], $_[2] )) }
 sub FETCHSIZE { length( $_[0]->[REF] ) / $_[0]->[SIZE] }
 sub EXISTS    { $_[1] < FETCHSIZE($_[0]) }
+sub EXTEND    { $_[0]->[REF] .= "\x0" x ($_[1] * $_[0]->[SIZE] - length $_[0]->[REF]) }
 sub STORESIZE {
 	( $_[1] > FETCHSIZE($_[0]) ) ?
 		(STORE($_[0], $_[1] - 1, 0)) :
-		(substr( $_[0]->[REF], $_[1] * $_[0]->[SIZE] ) = '' )
+		(CORE::substr( $_[0]->[REF], $_[1] * $_[0]->[SIZE] ) = '' )
 }
 sub DELETE    { warn "This array does not implement delete functionality" }
 
+package Prima::rect;
+
+sub new       { bless [$#_ ? (($#_ == 4) ? @_[1..$#_] : (0,0,@_[1,2])) : (0,0,0,0)], $_[0] }
+sub new_box   { bless [@_[1,2], $_[1] + $_[3] + 1, $_[2] + $_[4] + 1], $_[0] }
+sub clone     { bless [@{$_[0]}], ref $_[0] }
+sub is_empty  { $_[0]->[0] == $_[0]->[2] && $_[0]->[1] == $_[0]->[3] }
+sub origin    { $_[0]->[0], $_[0]->[1] }
+sub size      { $_[0]->[2] - $_[0]->[0] - 1, $_[0]->[3] - $_[0]->[1] - 1 }
+sub box       { $_[0]->[0], $_[0]->[1], $_[0]->[2] - $_[0]->[0] - 1, $_[0]->[3] - $_[0]->[1] - 1 }
+sub inclusive { $_[0]->[0], $_[0]->[1], $_[0]->[2] - 1, $_[0]->[3] - 1 }
+
+sub is_equal
+{
+	my ( $x, $y ) = @_;
+	if ( $x-> is_empty ) {
+		return $y->is_empty;
+	} elsif ( $y-> is_empty ) {
+		return 0;
+	} else {
+		return
+			$x->[0] == $y->[0] &&
+			$x->[1] == $y->[1] &&
+			$x->[2] == $y->[2] &&
+			$x->[3] == $y->[3];
+	}
+}
+
+sub union
+{
+	my ( $x, $y ) = @_;
+	return $y->clone if $x->is_empty;
+	return $x->clone if $y->is_empty;
+
+	$x = $x->clone;
+	$x->[0] = $y->[0] if $x->[0] > $y->[0];
+	$x->[1] = $y->[1] if $x->[1] > $y->[1];
+	$x->[2] = $y->[2] if $x->[2] < $y->[2];
+	$x->[3] = $y->[3] if $x->[3] < $y->[3];
+	return $x;
+}
+
+sub intersect
+{
+	my ( $x, $y ) = @_;
+	return ref($x)->new if
+		$x->is_empty or
+		$y->is_empty or
+		$x->[0] > $y->[2] or
+		$x->[2] < $y->[0] or
+		$x->[1] > $y->[3] or
+		$x->[3] < $y->[1]
+		;
+
+	$x = $x->clone;
+	$x->[0] = $y->[0] if $x->[0] > $y->[0];
+	$x->[1] = $y->[1] if $x->[1] > $y->[1];
+	$x->[2] = $y->[2] if $x->[2] < $y->[2];
+	$x->[3] = $y->[3] if $x->[3] < $y->[3];
+	return $x;
+}
+
+sub enlarge
+{
+	my ( $x, $d ) = @_;
+	return ref($x)->new if $x->is_empty;
+	$x = $x->clone;
+	$x->[$_] -= $d     for 0,1;
+	$x->[$_] += 2 * $d for 2,3;
+	return $x;
+}
+
+sub shrink { $_[0]->enlarge( -$_[1] ) }
+
 # class Object; base class of all Prima classes
 package Prima::Object;
 use vars qw(@hooks);
@@ -170,6 +288,22 @@ GENPROC
 
 sub DESTROY {}
 
+sub _debug
+{
+	my $f = shift;
+	printf STDERR "%d.%s [%dx%d]",
+		@{$f}{qw(size name width height)};
+	print STDERR " thin"   if $f->{style} & fs::Thin;
+	print STDERR " bold"   if $f->{style} & fs::Bold;
+	print STDERR " italic" if $f->{style} & fs::Italic;
+	print STDERR " fixed"  if $f->{pitch} == fp::Fixed;
+	print STDERR " B"      if $f->{vector} == fv::Bitmap;
+
+	printf STDERR " dir=%g", $f->{direction} if $f->{direction} != 0.0;
+	printf STDERR " %s",     $f->{encoding};
+	print STDERR "\n";
+}
+
 package Prima::Component;
 use vars qw(@ISA);
 @ISA = qw(Prima::Object);
@@ -246,6 +380,8 @@ sub get_notify_sub
 	return undef;
 }
 
+sub AUTOLOAD_DEEP_LOOKUP { $#_ ? $_[0]->{_autoload_deep_lookup} = $_[1] : $_[0]->{_autoload_deep_lookup} }
+
 sub AUTOLOAD
 {
 	no strict;
@@ -254,22 +390,13 @@ sub AUTOLOAD
 	Carp::confess "There is no such thing as \"$expectedMethod\"\n"
 		if scalar(@_) or not ref $self;
 	my ($componentName) = $expectedMethod =~ /::([^:]+)$/;
-	my $component = $self-> bring( $componentName);
-	Carp::croak("Unknown widget or method \"$expectedMethod\"")
+	my $component = $self-> bring( $componentName, $self->{_autoload_deep_lookup} ? 1000 : 0);
+	Carp::confess("Unknown widget or method \"$expectedMethod\"")
 		unless $component && ref($component);
 	return $component;
 }
 
-sub find_component
-{
-	my ( $self, $name ) = @_;
-	my @q = $self-> get_components;
-	while ( my $x = shift @q ) {
-		return $x if $x-> name eq $name;
-		push @q, $x-> get_components;
-	}
-	return undef;
-}
+sub find_component { $_[0]->bring($_[1], -1000) }
 
 package Prima::File;
 use vars qw(@ISA);
@@ -286,7 +413,6 @@ my %RNT = (
 sub notification_types { return \%RNT; }
 }
 
-
 sub profile_default
 {
 	my $def = $_[ 0]-> SUPER::profile_default;
@@ -300,6 +426,12 @@ sub profile_default
 	return $def;
 }
 
+sub profile_check_in
+{
+	my ( $self, $p, $default) = @_;
+	$p->{fd} = fileno($p->{file}) if exists $p->{file} && ! exists $p->{fd};
+}
+
 package Prima::Clipboard;
 use vars qw(@ISA);
 @ISA = qw(Prima::Component);
@@ -323,9 +455,9 @@ sub has_format
 
 sub copy
 {
-	my ( $self, $format, $data ) = @_;
+	my ( $self, $format, $data, $keep ) = @_;
 	$self-> open;
-	$self-> clear;
+	$self-> clear unless $keep;
 	$::application-> notify( 'Copy', $format, $self, $data );
 	$self-> close;
 }
@@ -338,8 +470,8 @@ sub paste
 	return $data;
 }
 
-sub text  { $#_ ? $_[0]->copy('Text',  $_[1]) : $_[0]->paste('Text') }
-sub image { $#_ ? $_[0]->copy('Image', $_[1]) : $_[0]->paste('Image') }
+sub text  { $#_ ? shift->copy('Text',  @_) : $_[0]->paste('Text') }
+sub image { $#_ ? shift->copy('Image', @_) : $_[0]->paste('Image') }
 
 package Prima::Region;
 use vars qw(@ISA);
@@ -387,13 +519,14 @@ sub image  { shift->bitmap_or_image( 'Prima::Image',        type => im::BW,
 package Prima::Drawable;
 use vars qw(@ISA);
 @ISA = qw(Prima::Component);
-use Prima::Bidi qw(is_bidi);
 use Prima::Drawable::Basic;
 
 sub profile_default
 {
 	my $def = $_[ 0]-> SUPER::profile_default;
 	my %prf = (
+		alpha           => 0xff,
+		antialias       => 0,
 		color           => cl::Black,
 		backColor       => cl::White,
 		fillMode        => fm::Overlay|fm::Alternate,
@@ -466,15 +599,6 @@ sub stretch_image {
 	) if $_[5]
 }
 
-sub text_out_bidi
-{
-	if ( $Prima::Bidi::enabled && is_bidi $_[1] ) {
-		return shift->text_out( Prima::Bidi::visual(shift), @_);
-	} else {
-		return shift->text_out(@_);
-	}
-}
-
 sub has_alpha_layer { 0 }
 
 sub spline
@@ -495,6 +619,18 @@ sub fillWinding # compatibility
 	$_[0]->fillMode(($_[1] ? fm::Winding : fm::Alternate) | fm::Overlay);
 }
 
+sub font_mapper { Prima::Font::Mapper->new( shift ) }
+
+sub graphic_context
+{
+	my $self = shift;
+	my $cb   = pop;
+	return unless $self->graphic_context_push;
+	$self->set(@_);
+	$cb->();
+	return $self->graphic_context_pop;
+}
+
 package Prima::Image;
 use vars qw( @ISA);
 @ISA = qw(Prima::Drawable);
@@ -600,9 +736,42 @@ sub ui_scale
 	return $self;
 }
 
+sub dataSize  { $_[0]->lineSize * $_[0]->height }
+sub scanline  { substr( $_[0]->data, $_[0]->lineSize * $_[1], $_[0]-> lineSize ) }
+sub shear     { $_[0]->transform(1,@_[2,1],1) }
 sub to_region { Prima::Region->new( image => shift ) }
 
-sub shear { $_[0]->transform(1,@_[2,1],1) }
+sub to_icon
+{
+	my ( $self, %set ) = @_;
+	my $fill = delete $set{fill};
+	my $type = delete $set{maskType} // 1;
+	my $and = Prima::Icon->new(
+		size => [ $self-> size ],
+		type => $type,
+	);
+	if ( defined $fill && length $fill ) {
+		$and->data( $fill  x ( $and->dataSize / (length $fill)));
+	}
+	return Prima::Icon->create_combined( $self, $and, %set );
+}
+
+sub load_stream
+{
+	shift;
+	require Prima::Image::base64;
+	my ($ok, $error) = Prima::Image::base64::load_icon(@_);
+	$@ = $error unless $ok;
+	return $ok;
+}
+
+sub save_stream
+{
+	require Prima::Image::base64;
+	my ( $ok, $error ) = Prima::Image::base64::save(@_);
+	$@ = $error unless $ok;
+	return $ok;
+}
 
 package Prima::Icon;
 use vars qw( @ISA);
@@ -632,6 +801,11 @@ sub profile_check_in
 	$self-> SUPER::profile_check_in( $p, $default);
 }
 
+sub maskLineSize { int(( $_[0]->width * $_[0]->maskType + 31 ) / 32 ) * 4 }
+sub maskSize     { $_[0]->maskLineSize * $_[0]->height }
+sub maskline     { substr( $_[0]->mask, $_[0]->maskLineSize * $_[1], $_[0]-> maskLineSize ) }
+sub maskImage    { my ( undef, $mask ) = shift->split; $mask }
+
 sub mirror
 {
         my ($self, $vertically) = @_;
@@ -643,8 +817,9 @@ sub mirror
 
 sub create_combined
 {
-	my $self = shift->new( autoMasking => am::None );
-	$self->combine(@_);
+	my ( $class, $xor, $and, @rest ) = @_;
+	my $self = $class->new( autoMasking => am::None, @rest );
+	$self->combine($xor, $and);
 	return $self;
 }
 
@@ -685,6 +860,15 @@ sub image
 	return $image;
 }
 
+sub load_stream
+{
+	shift;
+	require Prima::Image::base64;
+	my ($ok, $error) = Prima::Image::base64::load_icon(@_);
+	$@ = $error unless $ok;
+	return $ok;
+}
+
 package Prima::DeviceBitmap;
 use vars qw( @ISA);
 @ISA = qw(Prima::Drawable);
@@ -718,7 +902,7 @@ sub profile_check_in
 
 sub has_alpha_layer { shift->type == dbt::Layered }
 
-sub dup 
+sub dup
 {
 	my $self = shift;
 	my $dup = ref($self)->new(
@@ -727,7 +911,7 @@ sub dup
 	);
 	$dup->backColor(0);
 	$dup->clear;
-	$dup->put_image(0,0,$self,rop::SrcOver);
+	$dup->put_image(0,0,$self,rop::Blend);
 	return $dup;
 }
 
@@ -897,6 +1081,17 @@ sub notification_types { return \%RNT; }
 	y_centered        => 0,
 );
 
+my $_markup_loaded;
+sub _markup($)
+{
+	unless ( $_markup_loaded ) {
+		eval "use Prima::Drawable::Markup;";
+		die $@ if $@;
+		$_markup_loaded++;
+	}
+	return Prima::Drawable::Markup::M( ${ $_[0] } );
+}
+
 sub profile_default
 {
 	my $def = $_[ 0]-> SUPER::profile_default;
@@ -929,6 +1124,10 @@ sub profile_check_in
 	$self-> SUPER::profile_check_in( $p, $default);
 	delete $p-> { font} unless defined $orgFont;
 
+	for my $tx ( qw(text hint)) {
+		$p->{$tx} = _markup $p->{$tx} if defined $p->{$tx} && (ref($p->{$tx}) // '') eq 'SCALAR';
+	}
+
 	my $name = defined $p-> {name} ? $p-> {name} : $default-> {name};
 	$p-> {text} = $name
 		if !defined $p-> { text} and !defined $default-> {text};
@@ -1133,6 +1332,18 @@ sub popupLight3DColor     { return shift-> popupColorIndex( ci::Light3DColor, @_
 sub x_centered       {($#_)?$_[0]-> set_centered(1,0)      :$_[0]-> raise_wo("x_centered"); }
 sub y_centered       {($#_)?$_[0]-> set_centered(0,1)      :$_[0]-> raise_wo("y_centered"); }
 
+sub hint
+{
+	return $_[0]->get_hint unless $#_;
+	$_[0]->set_hint( (( ref($_[1]) // '') eq 'SCALAR') ? _markup $_[1] : $_[1] );
+}
+
+sub text
+{
+	return $_[0]->get_text unless $#_;
+	$_[0]->set_text( (( ref($_[1]) // '') eq 'SCALAR') ? _markup $_[1] : $_[1] );
+}
+
 sub insert
 {
 	my $self = shift;
@@ -1270,7 +1481,6 @@ sub rect_bevel
 
 	return $canvas-> rect3d( $x, $y, $x1, $y1, $width, @c3d, $fill)
 		if $width < 2;
-	my $back  = (defined($fill) && !ref($fill)) ? $fill : $self-> backColor;
 
 	# 0 - upper left under 2 -- inner square
 	# 1 - lower right over 3
@@ -1355,7 +1565,7 @@ sub begin_drag
 		my $fh    = $self->font->height;
 		my @sz = ( 0, 10 + $fh * @lines );
 		for my $text ( @lines ) {
-			my $tw = $self->get_text_width($text, 1);
+			my $tw = $self->get_text_shape_width($text, 1);
 			$sz[0] = $tw if $sz[0] < $tw;
 		}
 		$sz[0] += 10;
@@ -1374,11 +1584,11 @@ sub begin_drag
 		$i->clear;
 		my $y = $i->height - $fh - 5;
 		for my $text ( @lines ) {
-			$i->text_out( $text, 5, $y);
+			$i->text_shape_out( $text, 5, $y);
 			$y -= $fh;
 		}
 		$i->end_paint;
-		$i->alpha(160, 0, 0, $i->size);
+		$i->bar_alpha(160, 0, 0, $i->size);
 		$opt{preview} = $i;
 	}
 
@@ -1428,7 +1638,7 @@ sub begin_drag
 				if $p->isa('Prima::Icon');
 			$n->put_image( 0, $p->height, $i, rop::SrcCopy);
 			$n->put_image( $i->width, 0, $p, rop::SrcCopy);
-			$n->alpha(0xff, $i->width, 0, $i->width + $p->width - 1, $p->height - 1)
+			$n->bar_alpha(0xff, $i->width, 0, $i->width + $p->width - 1, $p->height - 1)
 				if !$p->isa('Prima::Icon');
 			$n->{__pointerHotSpot} = \@hs;
 			$pointers{$action} = $n;
@@ -1450,6 +1660,7 @@ sub begin_drag
 	if ( $self->alive ) {
 		if ( $ret == dnd::None && $opt{preview} ) {
 			my @npp = $::application->pointerPos;
+			$npp[1] -= $opt{preview}->height;
 			my $paint_flag = 0;
 			my $flyback = Prima::Widget->new(
 				size      => [ $opt{preview}->size ],
@@ -1463,18 +1674,27 @@ sub begin_drag
 					$paint_flag = 1;
 				}
 			);
+			$flyback-> insert( Timer => 
+				onTick => sub {
+					$flyback->destroy if $flyback;
+					undef $flyback;
+				},
+				timeout => 1000,
+			)-> start;
 			$flyback->bring_to_front;
 			my @targ = map { $_ / 2 } $flyback->size;
 			while (abs( $npp[0] - $opp[0]) > $targ[0] || abs($npp[1] - $opp[1]) > $targ[1]) {
-				@npp = map { ( $npp[$_] + $opp[$_] ) / 2 } 0, 1;
+				@npp = map { $npp[$_] * 0.8 + $opp[$_] * 0.2 } 0, 1;
 				my $max_wait = 10;
 				$::application->yield while !$paint_flag && $max_wait--;
+				last unless $flyback;
 				$paint_flag = 0;
-				CORE::select(undef, undef, undef, 0.1);
+				CORE::select(undef, undef, undef, 0.02);
 				$flyback->origin(@npp);
 				$flyback->bring_to_front;
 			}
-			$flyback->destroy;
+			$flyback->destroy if $flyback;
+			undef $flyback;
 		}
 		$self->pointer($pointer); # dnd_start doesn't affect children pointers and doesn't restore them
 		$self->remove_notification($_) for @id;
@@ -1824,20 +2044,46 @@ sub on_paint
 	my ( $x, $y) = ( 3, $size[1] - 1 - $fh);
 	my @ln = $canvas->text_split_lines($self->text);
 	for ( @ln) {
-		$canvas-> text_out_bidi( $_, $x, $y);
+		$canvas-> text_shape_out( $_, $x, $y);
 		$y -= $fh;
 	}
 }
 
-sub text
+sub set_text
 {
-	return $_[0]-> SUPER::text unless $#_;
 	my $self = $_[0];
-	$self-> SUPER::text( $_[1]);
+	$self-> SUPER::set_text( $_[1]);
 	$self-> notify( 'Change');
 	$self-> repaint;
 }
 
+package Prima::Font::Mapper;
+
+sub new { bless { canvas => $_[1] }, $_[0] }
+
+sub get   { Prima::Application->font_mapper_action( command => 'get_font', index => $_[1] ) }
+sub count { Prima::Application->font_mapper_action( command => 'get_count' ) }
+
+sub index
+{
+	my $canvas = $_[0]->{canvas};
+	return undef unless $canvas;
+	return Prima::Application->font_mapper_action( command => 'get_index', font => $canvas->font );
+}
+
+sub AUTOLOAD
+{
+	no strict;
+	my $self = shift;
+	my $cmd = $AUTOLOAD;
+	$cmd =~ s/.*://;
+	my %font = ( style => fs::Normal, @_ );
+	Carp::carp("at least font name is required"), return -1 unless exists $font{name};
+	return Prima::Application->font_mapper_action( command => $cmd, font => \%font );
+}
+
+sub DESTROY {}
+
 package Prima::Application;
 use vars qw(@ISA @startupNotifications);
 @ISA = qw(Prima::Widget);
@@ -1846,6 +2092,7 @@ use vars qw(@ISA @startupNotifications);
 my %RNT = (
 	%{Prima::Widget-> notification_types()},
 	FormatExists => nt::Action,
+	Clipboard    => nt::Action,
 	Copy         => nt::Action,
 	Paste        => nt::Action,
 	Idle         => nt::Default,
@@ -1863,6 +2110,7 @@ sub profile_default
 		autoClose      => 0,
 		pointerType    => cr::Arrow,
 		pointerVisible => 1,
+		language       => Prima::Application->get_system_info->{guiLanguage},
 		icon           => undef,
 		owner          => undef,
 		scaleChildren  => 0,
@@ -1882,8 +2130,9 @@ sub profile_default
 		printerModule  => $unix ? 'Prima::PS::Printer' : '',
 		helpClass      => 'Prima::HelpViewer',
 		helpModule     => 'Prima::HelpViewer',
+		textDirection  => 0,
 		uiScaling      => 0,
-		wantUnicodeInput => 0,
+		wantUnicodeInput => 1,
 	);
 	@$def{keys %prf} = values %prf;
 	return $def;
@@ -1892,6 +2141,7 @@ sub profile_default
 sub profile_check_in
 {
 	my ( $self, $p, $default) = @_;
+	$p->{textDirection} //= $self->lang_is_rtl($p->{language} // $default->{language});
 	$self-> SUPER::profile_check_in( $p, $default);
 	delete $p-> { printerModule};
 	delete $p-> { owner};
@@ -1926,8 +2176,8 @@ sub setup
 	# setup image cliboard transfer routines specific to gtk
 	if ( $unix ) {
 		my %weights = (
-			bmp  => 4,  # bmp is independent on codecs
-			png  => 3,  # png is lossless
+			png  => 4,  # png is lossless
+			bmp  => 3,  # bmp is independent on codecs but huge
 			tiff => 2,  # tiff is usually lossless
 		);
 		my %codecs  = map { lc($_-> {fileShortType})  => $_ } @{Prima::Image-> codecs};
@@ -1935,6 +2185,7 @@ sub setup
 		my @codecs = map { {
 			mime => "image/$_",
 			id   => $codecs{$_}->{codecID},
+			w    => $codecs{$_}->{weight},
 		} } sort { $codecs{$b}->{weight} <=> $codecs{$a}->{weight} } keys %codecs;
 		my $clipboard = $self-> Clipboard;
 		$clipboard-> register_format($_->{mime}) for @codecs;
@@ -1960,7 +2211,7 @@ sub get_printer
 			eval 'use ' . $_[0]-> {PrinterModule} . ';';
 			die "$@" if $@;
 		}
-		$_[0]-> {Printer} = $_[0]-> {PrinterClass}-> create( owner => $_[0]);
+		$_[0]-> {Printer} = $_[0]-> {PrinterClass}-> create( owner => $_[0], system => 1);
 	}
 	return $_[0]-> {Printer};
 }
@@ -1969,6 +2220,31 @@ sub hintFont      {($#_)?$_[0]-> set_hint_font        ($_[1])  :return Prima::Fo
 sub helpModule    {($#_)?$_[0]-> {HelpModule} = $_[1] : return $_[0]-> {HelpModule}}
 sub helpClass     {($#_)?$_[0]-> {HelpClass}  = $_[1] : return $_[0]-> {HelpClass}}
 
+sub lang_is_rtl
+{
+	my $lang = $_[1] // $_[0]->get_system_info->{guiLanguage};
+	$lang =~ /^(
+		ar| # arabic
+		dv| # divehi
+		fa| # persian (farsi)
+		ha| # hausa
+		he| # hebrew
+		iw| # hebrew (old code)
+		ji| # yiddish (old code)
+		ps| # pashto, pushto
+		ur| # urdu
+		yi  # yiddish
+	)/x ? 1 : 0
+}
+
+sub language
+{
+	return $_[0]->{language} unless $#_;
+	my ( $self, $lang ) = @_;
+	$self->{language} = $lang;
+	$self->textDirection( $_[0]-> lang_is_rtl($lang));
+}
+
 sub help_init
 {
 	return 0 unless length $_[0]-> {HelpModule};
@@ -1991,24 +2267,34 @@ sub open_help
 	return $self-> {HelpClass}-> open($link);
 }
 
+sub on_clipboard
+{
+	my ( $self, $clipboard, $action, $target ) = @_;
+	if ($clipboard->format_exists('Image')) {
+		if ( my ( $codec ) = grep { $target eq $_->{mime} } @{ $self-> {GTKImageClipboardFormats} // [] }) {
+			my ($bits, $handle) = ('');
+			my $i = $clipboard->fetch('Image') or return;
+			if (open( $handle, '>', \$bits) and $i->save($handle, codecID => $codec->{id})) {
+				$clipboard->store($codec->{mime}, $bits);
+			}
+		}
+	}
+}
+
 sub on_copy
 {
 	my ( $self, $format, $clipboard, $data ) = @_;
 	$clipboard-> store( $format, $data);
 	if ( $format eq 'Image') {
-		if ( my $formats = $self-> {GTKImageClipboardFormats} ) {
-			my ($bmp, $bits, $handle) = ($formats->[0], '');
-			if (open( $handle, '>', \$bits) and $data->save($handle, codecID => $bmp->{id})) {
-				$clipboard->store($bmp->{mime}, $bits);
-			}
-		}
+		# store(undef) is a special flag for x11 when data can be provided on demand for this format
+		$clipboard->store($_, undef) for map { $_->{mime} } @{ $self-> {GTKImageClipboardFormats} // [] };
 	}
 }
 
 sub on_formatexists
 {
 	my ( $self, $format, $clipboard, $ref) = @_;
-	
+
 	if ( $format eq 'Text') {
 		if ( $self-> wantUnicodeInput) {
 			return $$ref = 'UTF8' if $clipboard-> format_exists( 'UTF8');
@@ -2030,24 +2316,17 @@ sub on_formatexists
 sub on_paste
 {
 	my ( $self, $format, $clipboard, $ref) = @_;
-	
+
 	if ( $format eq 'Text') {
 		if ( $self-> wantUnicodeInput) {
 			return if defined ( $$ref = $clipboard-> fetch( 'UTF8'));
 		}
 		$$ref = $clipboard-> fetch( 'Text');
 	} elsif ( $format eq 'Image') {
-		$$ref = $clipboard-> fetch( 'Image');
-		return if defined $$ref;
-
-		my $codecs = $self-> {GTKImageClipboardFormats};
-		return unless $codecs;
-
-		my %formats = map { $_ => 1 } $clipboard-> get_formats;
-		my @codecs  = grep { $formats{$_->{mime}} } @$codecs;
-		return unless @codecs;
-
-		my $data = $clipboard-> fetch($codecs[0]->{mime});
+		my $codecs  = $self-> {GTKImageClipboardFormats} or goto DEFAULT;
+		my %formats = map  { $_ => 1 } $clipboard-> get_formats;
+		my @codecs  = grep { $formats{$_->{mime}} && $_->{w} > 1 } @$codecs or goto DEFAULT;
+		my $data    = $clipboard-> fetch($codecs[0]->{mime});
 		return unless defined $data;
 
 		my $handle;
@@ -2056,7 +2335,8 @@ sub on_paste
 		local $@;
 		$$ref = Prima::Image-> load($handle, loadExtras => 1 );
 	} else {
-		$$ref = $clipboard-> fetch( $format);
+	DEFAULT:
+       		$$ref = $clipboard-> fetch( $format);
 	}
 	undef;
 }
