@@ -8,11 +8,11 @@ package Prima::Outlines;
 use strict;
 use warnings;
 use Cwd;
-use Prima qw(Classes IntUtils StdBitmap);
+use Prima qw(StdBitmap);
 
 package Prima::OutlineViewer;
-use vars qw(@ISA @images @imageSize);
-@ISA = qw(Prima::Widget Prima::MouseScroller Prima::GroupScroller Prima::ListBoxUtils);
+use vars qw(@images @imageSize);
+use base qw(Prima::Widget Prima::Widget::MouseScroller Prima::Widget::GroupScroller Prima::Widget::ListBoxUtils);
 
 use constant DATA     => 0;
 use constant DOWN     => 1;
@@ -310,6 +310,7 @@ sub on_paint
 		fillPattern => fp::SimpleDots,
 		color       => cl::White,
 		backColor   => cl::Black,
+		rop2        => rop::CopyPut,
 	);
 
 
@@ -415,6 +416,7 @@ sub on_paint
 		fillPattern => fp::Solid,
 		color       => $clr[0],
 		backColor   => $clr[1],
+		rop2        => rop::NoOper,
 	);
 
 #
@@ -1723,7 +1725,7 @@ my $unix = Prima::Application-> get_system_info-> {apc} == apc::Unix || $^O =~ /
 my @images;
 my @drvImages;
 
-{
+unless ($^C) {
 	my $i = 0;
 	my @idx = (  sbmp::SFolderOpened, sbmp::SFolderClosed);
 	$images[ $i++] = Prima::StdBitmap::icon( $_) for @idx;
@@ -2153,8 +2155,8 @@ node structure is used as a reference. It is important to use a valid reference
 since the class does not always perform the check if the node belongs to internal node list due to
 the speed reasons.
 
-C<Prima::OutlineViewer> is a descendant of C<Prima::GroupScroller> and C<Prima::MouseScroller>,
-so some properties and methods are not described here. See L<Prima::IntUtils> for these.
+C<Prima::OutlineViewer> is a descendant of C<Prima::Widget::GroupScroller> and C<Prima::Widget::MouseScroller>,
+so some properties and methods are not described here. 
 
 The class is not usable directly.
 
@@ -2635,6 +2637,6 @@ Dmitry Karasik, E<lt>dmitry@karasik.eu.orgE<gt>.
 
 =head1 SEE ALSO
 
-L<Prima>, L<Prima::Widget>, L<Prima::IntUtils>, <examples/outline.pl>.
+L<Prima>, L<Prima::Widget>, <examples/outline.pl>.
 
 =cut
