@@ -94,8 +94,10 @@ use constant MDIV_TAG      => 2;
 use constant MDIV_STYLE    => 3;
 
 # model entries
-use constant T_NORMAL          => 0;
-use constant T_DIV             => 1;
+use constant T_NORMAL          => 0x010;
+use constant T_DIV             => 0x020;
+use constant T_TYPE_MASK       => 0x030;
+use constant T_LOOKAHEAD       => 0x007;
 use constant TDIVTAG_OPEN      => 0;
 use constant TDIVTAG_CLOSE     => 1;
 use constant TDIVSTYLE_SOLID   => 0;
@@ -119,7 +121,7 @@ sub model_create
 {
 	my %opt = @_;
 	return (
-		$opt{type}   // T_NORMAL,
+		T_NORMAL | (($opt{lookahead} // 0) & T_LOOKAHEAD),
 		$opt{offset} // 0,
 		$opt{indent} // 0,
 		$opt{font}   // 0
@@ -179,7 +181,8 @@ sub profile_default
 		],
 		pageName      => '',
 		topicView     => 0,
-		textDirection  => $::application->textDirection,
+		textDirection => $::application->textDirection,
+		justify       => 1,
 	);
 	@$def{keys %prf} = values %prf;
 	return $def;
@@ -199,6 +202,7 @@ sub init
 	$self-> {topics}     = [];
 	$self-> {hasIndex}   = 0;
 	$self-> {topicView}  = 0;
+	$self-> {justify}  = 0;
 	$self-> {lastLinkPointer} = -1;
 	my %profile = $self-> SUPER::init(@_);
 
@@ -209,7 +213,7 @@ sub init
 	$self-> {fontPalette}-> [1] = \%font;
 	$self-> {fontPaletteSize} = 2;
 
-	$self-> $_($profile{$_}) for qw( styles images pageName topicView);
+	$self-> $_($profile{$_}) for qw( styles images pageName topicView justify);
 
 	return %profile;
 }
@@ -477,6 +481,15 @@ sub topicView
 	$self-> load_file( $self-> {pageName});
 }
 
+sub justify
+{
+	return $_[0]-> {justify} unless $#_;
+	my ( $self, $justify) = @_;
+	$justify = ( $justify ? 1 : 0);
+	return if $self-> {justify} == $justify;
+	$self-> {justify} = $justify;
+	$self-> format(1);
+}
 
 sub pageName
 {
@@ -703,7 +716,8 @@ sub add_image
 	$w *= 72 / $resolution[0];
 	$h *= 72 / $resolution[1];
 	$src-> {stretch} = [$w, $h];
-	$self-> {readState}-> {pod_cutting} = $opt{cut} ? 0 : 1
+	my $r = $self-> {readState};
+	$r-> {pod_cutting} = $opt{cut} ? 0 : 1
 		if defined $opt{cut};
 
 	my @imgop = (
@@ -716,18 +730,20 @@ sub add_image
 	);
 
 	push @{$self-> {model}},
-		$opt{title} ? [div_create(open => 1, style => TDIVSTYLE_OUTLINE)] : (),
+		$opt{title} ? [div_create(
+			open     => 1,
+			style    => TDIVSTYLE_OUTLINE,
+		)] : (),
 		[model_create(
-			indent => $self-> {readState}-> {indent},
-			offset => $self-> {readState}-> {bigofs}
-		),
-		@imgop],
+			indent    => $r-> {indent},
+			offset    => $r-> {bigofs},
+			lookahead => $opt{title} ? 4 : 1,
+		), @imgop],
 		;
-	if ( $opt{title}) {
-		my $r = $self-> {readState};
 
+	if ( $opt{title}) {
 		my @g = model_create(
-			indent => $self-> {readState}-> {indent},
+			indent => $r-> {indent},
 			offset => $r-> {bigofs}
 		);
 		push @g,
@@ -793,12 +809,12 @@ sub _imgpaint
 	$dx *= $res[0] / 72;
 	$dy *= $res[1] / 72;
 	$canvas-> stretch_image( $x, $y, $dx, $dy, $img);
-	if ( $self-> {selectionPaintMode}) {
-		my @save = ( fillPattern => $canvas-> fillPattern, rop => $canvas-> rop, fillPatternOffset => [$canvas->fillPatternOffset]);
-		$canvas-> set( fillPattern => fp::Borland, rop => rop::AndPut, fillPatternOffset => [$x, $y]);
-		$canvas-> bar( $x, $y, $x + $dx - 1, $y + $dy - 1);
-		$canvas-> set( @save);
-	}
+	$canvas-> graphic_context(
+		color             => $canvas->backColor,
+		fillPattern       => fp::SimpleDots,
+		fillPatternOffset => [$x, $y],
+		sub { $canvas-> bar( $x, $y, $x + $dx - 1, $y + $dy - 1) }
+	) if $self-> {selectionPaintMode};
 }
 
 sub _bulletpaint
@@ -962,6 +978,7 @@ sub close_read
 
 	my $secid = 0;
 	my $msecid = scalar(@{$self-> {topics}});
+	delete $self->{index_ends_at};
 
 	unless ( $msecid) {
 		push @{$self-> {topics}}, [
@@ -1033,7 +1050,10 @@ sub close_read
 	$$_[M_TEXT_OFFSET] += $offsets[0]      for @$m[0..$text_ends_at[1]-1];
 	$$_[M_TEXT_OFFSET] -= $text_ends_at[0] for @$m[$text_ends_at[1]..$index_ends_at[1]-1];
 	# next reshuffle the model
-	unshift @$m, splice( @$m, $text_ends_at[1]);
+	my @index_section = splice( @$m, $text_ends_at[1]);
+	unshift @$m, @index_section;
+	$self->{index_ends_at} = @index_section;
+	undef @index_section;
 	# text
 	my $t = $self-> {text};
 	my $ts = substr( $$t, $text_ends_at[0]);
@@ -1119,7 +1139,18 @@ sub add
 	return unless $r;
 
 	$p =~ s/\n//g;
-	my $g = [ model_create( indent => $indent, offset => $r-> {bigofs}) ];
+	my $g = [ model_create(
+		indent    => $indent,
+		offset    => $r-> {bigofs},
+		# #0 =head1   =item TEXT  =item *
+		# #1 .        .           .
+		# #2 text     text
+		# #3 .        .
+		lookahead =>
+			($style == STYLE_ITEM && $r->{bulletMode}) ? 1 : (
+				(($style >= STYLE_HEAD_1) && ($style <= STYLE_ITEM)) ? 3 : 0
+			),
+	) ];
 	my $styles = $self-> {styles};
 	my $no_push_block;
 	my $itemid = scalar @{$self-> {model}};
@@ -1158,7 +1189,6 @@ sub add
 		$p =~ s/[\s\t]+/ /g;
 		$p =~ s/([\200-\377])/"E<".ord($1).">"/ge;
 		$p =~ s/(E<[^<>]+>)/noremap($1)/ge;
-		$p =~ s/([:A-Za-z_][:A-Za-z_0-9]*\([^\)]*\))/C<$1>/g;
 		my $maxnest = 10;
 		my $linkStart = scalar @{$self-> {links}};
 		my $m = $p;
@@ -1335,9 +1365,6 @@ sub add
 			my $pp = $p;
 			$pp =~ s/\|//g;
 			$pp =~ s/([<>])/'E<' . (($1 eq '<') ? 'lt' : 'gt') . '>'/ge;
-			if ( $style == STYLE_ITEM && $pp =~ /^\s*[a-z]/) {
-				$pp =~ s/([\s\)\(\[\]\{\}].*)$/C<$1>/; # seems like function entry?
-			}
 			my $newTopic = [ $itemid, 0, $pp, $style, $itemDepth, $linkStart];
 			$self-> _close_topic( $style, $newTopic);
 			push @{$self-> {topics}}, $newTopic;
@@ -1491,17 +1518,22 @@ sub paint_code_div
 	my ( $self, $canvas, $block, $state, $x, $y, $coord) = @_;
 	my $f  = $canvas->font;
 	my ($style, $w, $h) = @$coord;
-	my @x = ( $canvas-> backColor, $canvas-> color );
-	my $path = $canvas->new_path->round_rect($x, $y, $x + $w, $y + $h, 20);
+	my %save = map { $_ => $canvas-> $_() } qw(color);
+
+	my $path = $canvas->new_path->round_rect(
+		$x, $y,
+		$x + $w, $y + $h,
+		$self->{defaultFontSize} * 2 * $self->{resolution}->[0] / 96.0
+	);
 	if ( $style == TDIVSTYLE_SOLID ) {
+		$save{backColor} = $canvas->backColor;
 		$canvas->set(backColor => $self->{colorMap}->[5], color => 0xcccccc);
 		$path->fill_stroke;
-		$canvas-> set( backColor => $x[0], color => $x[1] );
 	} else {
 		$canvas->set(color => 0x808080);
-		$path-> stroke;
-		$canvas-> set( color => $x[1] );
+		$path->stroke;
 	}
+	$canvas-> set(%save);
 }
 
 sub add_code_div
@@ -1552,7 +1584,7 @@ sub format_chunks
 		my $g = tb::block_create();
 		my $m = $self-> {model}-> [$mid];
 
-		if ( $m->[M_TYPE] == T_DIV ) {
+		if (( $m->[M_TYPE] & T_TYPE_MASK) == T_DIV ) {
 			if ( $m->[MDIV_TAG] == TDIVTAG_OPEN) {
 				$f->{verbatim} = scalar @{ $self->{blocks} };
 			} else {
@@ -1573,7 +1605,7 @@ sub format_chunks
 
 		my $next_text_offs = ( $mid == $#{$self->{model}} ) ? length( ${$self->{text}} ) : $self->{model}->[$mid + 1]->[M_TEXT_OFFSET];
 		my $indent = $$m[M_INDENT] * $$indents[ $$m[M_FONT_ID]];
-		@blocks = $self-> block_wrap( $self, $g, $state, $formatWidth - $indent);
+		@blocks = $self-> format_block( $self, $g, $state, $formatWidth, $indent);
 
 		# adjust size
 		for ( @blocks) {
@@ -1681,13 +1713,25 @@ sub format_chunks
 
 sub print
 {
-	my ( $self, $canvas, $callback) = @_;
+	my ( $self, $canvas, $_callback) = @_;
 
 	my ( $min, $max, $linkIdStart) = @{$self-> {modelRange}};
 	return 1 if $min >= $max;
 	my $ret = 0;
 
-	goto ABORT if $callback && ! $callback-> ();
+	my $save_defaultFontSize = $self->{defaultFontSize};
+	my $print_defaultFontSize = 10;
+	$self->{defaultFontSize} = $print_defaultFontSize;
+
+	my $callback = sub {
+		return 1 unless $_callback;
+		$self->{defaultFontSize} = $save_defaultFontSize;
+		my $ret = $_callback->();
+		$self->{defaultFontSize} = $print_defaultFontSize;
+		return $ret;
+	};
+
+	goto ABORT unless $callback-> ();
 
 	# cache indents
 	my @indents;
@@ -1703,7 +1747,7 @@ sub print
         my $hmargin = $formatWidth  / 24;
         my $vmargin = $formatHeight / 12;
         $formatWidth  -= $hmargin * 2;
-        $formatHeight -= $vmargin * 2;
+        $formatHeight -= $vmargin * 1.5;
         $canvas->translate( $hmargin, $vmargin );
 
 	my $mid = $min;
@@ -1720,47 +1764,81 @@ sub print
 		$canvas->font(\%save);
 		$pageno++;
 	};
+
 	my $new_page = sub {
-		goto ABORT if $callback && ! $callback-> ();
+		goto ABORT unless $callback-> ();
 		$pagenum->();
 		goto ABORT unless $canvas-> new_page;
 		$canvas->translate( $hmargin, $vmargin );
 	};
 
-	for ( ; $mid <= $max; $mid++) {
-		my $g = tb::block_create();
-		my $m = $self-> {model}-> [$mid];
-		next if $$m[M_TYPE] != T_NORMAL; # don't print div background
+	my $get_blocks = sub {
+		# don't print the Index section
+		return if defined $self->{index_ends_at} and $mid < $self->{index_ends_at};
 
-		my @blocks;
+		my $m = $self-> {model}-> [$mid];
+		return if ($$m[M_TYPE] & T_TYPE_MASK) != T_NORMAL; # don't print divs
+
+		my $g = tb::block_create();
 		$$g[ tb::BLK_TEXT_OFFSET] = $$m[M_TEXT_OFFSET];
 		$$g[ tb::BLK_Y] = undef;
 		push @$g, @$m[ M_START .. $#$m ];
 
 		# format the paragraph
 		my $indent = $$m[M_INDENT] * $indents[ $$m[M_FONT_ID]];
-		@blocks = $self-> block_wrap( $canvas, $g, $state, $formatWidth - $indent);
+		return $self-> format_block( $canvas, $g, $state, $formatWidth, $indent, 1);
+	};
+
+	my @block_queue;
+	for ( ; $mid <= $max; $mid++) {
+		my @queue;
+		my @blocks = $get_blocks->() or next;
+		my $m      = $self-> {model}-> [$mid];
+		push @queue, [ $m, \@blocks ];
+
+		# try to look-ahead some blocks, see if they all can fit on the same page
+		if ( my $lookahead = $m->[M_TYPE] & T_LOOKAHEAD) {
+			my $y2 = 0;
+			$y2 += $$_[tb::BLK_HEIGHT] for @blocks;
+			for ( 1 .. $lookahead ) {
+				last if $mid + 1 > $max;
+				my $xm = $self-> {model}-> [$mid + 1];
+				last if $xm->[M_TYPE] & T_LOOKAHEAD;
+				$mid++;
+				my @b = $get_blocks->();
+				$y2 += $$_[tb::BLK_HEIGHT] for @b;
+				push @queue, [ $xm, \@b ] if @b;
+			}
+			if ( $y2 > $y && @blocks && $blocks[0][tb::BLK_HEIGHT] <= $formatHeight ) {
+				$new_page->();
+				$y = $formatHeight;
+			}
+		}
+
 
 		# paint
 		$self-> reset_state;
-		for ( @blocks) {
-			my $b = $_;
-			if ( $y < $$b[ tb::BLK_HEIGHT]) {
-				if ( $$b[ tb::BLK_HEIGHT] < $formatHeight) {
-					$new_page->();
-					$y = $formatHeight - $$b[ tb::BLK_HEIGHT];
-					$self-> block_draw( $canvas, $b, $indent, $y);
+		for my $q (@queue) {
+			my ( $m, $blocks ) = @$q;
+			my $indent = $$m[M_INDENT] * $indents[ $$m[M_FONT_ID]];
+			for my $b ( @$blocks) {
+				if ( $y < $$b[ tb::BLK_HEIGHT]) {
+					if ( $$b[ tb::BLK_HEIGHT] < $formatHeight) {
+						$new_page->();
+						$y = $formatHeight - $$b[ tb::BLK_HEIGHT];
+						$self-> block_draw( $canvas, $b, $indent, $y);
+					} else {
+						$y -= $$b[ tb::BLK_HEIGHT];
+						while ( $y < 0) {
+							$new_page->();
+							$self-> block_draw( $canvas, $b, $indent, $y);
+							$y += $formatHeight;
+						}
+					}
 				} else {
 					$y -= $$b[ tb::BLK_HEIGHT];
-					while ( $y < 0) {
-						$new_page->();
-						$self-> block_draw( $canvas, $b, $indent, $y);
-						$y += $formatHeight;
-					}
+					goto ABORT unless $self-> block_draw( $canvas, $b, $indent, $y);
 				}
-			} else {
-				$y -= $$b[ tb::BLK_HEIGHT];
-				goto ABORT unless $self-> block_draw( $canvas, $b, $indent, $y);
 			}
 		}
 	}
@@ -1768,9 +1846,36 @@ sub print
 
 	$ret = 1;
 ABORT:
+	$self->{defaultFontSize} = $save_defaultFontSize;
 	return $ret;
 }
 
+sub format_block
+{
+	my ( $self, $canvas, $block, $state, $width, $indent, $printing ) = @_;
+
+	$width -= $indent * 2;
+
+	my @blocks = $self-> block_wrap( $canvas, $block, $state, $width ) or return;
+
+	if ( $printing and 1 == @blocks and $width < $blocks[0][tb::BLK_WIDTH] ) {
+		# cannot wrap a (verbatim?) block -- force break it
+		return $self-> block_wrap( $canvas, $block, $state, $width, stripLeadingSpaces => 0, ignoreWraps => 1);
+	}
+
+	if ( $self->{justify}) {
+		my @b;
+		for ( my $i = 0; $i < $#blocks; $i++) {
+			my $b = $self->justify_interspace( $canvas, $blocks[$i], $width);
+			push @b, $b // $blocks[$i];
+		}
+		push @b, $blocks[-1];
+		@blocks = @b;
+	}
+
+	return @blocks;
+}
+
 sub select_text_offset
 {
 	my ( $self, $pos) = @_;
